
#include "dmxoe_codec.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

bool ready (int sock)
{
  struct timeval nowait = {0, 0};
  int status;
  fd_set set;
  FD_ZERO(&set);
  FD_SET(sock, &set);
  status = select(sock+1, &set, NULL, NULL, &nowait);
  if(status == -1)
    perror("select error"), exit(1);
  return status > 0;
}


int recv_udp(U16 port_in, U8 pktbuf[], unsigned pktlen)
{
  static int sock = -1;
  struct sockaddr_in local;
  int n = sizeof(local);
  if(sock == -1)
    {
      sock = socket(AF_INET, SOCK_DGRAM, 0);
      if(sock < 0)
        perror("socket error"), exit(1);
      bzero(&local, sizeof(local));
      local.sin_family = AF_INET;
      local.sin_port = htons(port_in);
      if(bind(sock, (struct sockaddr*)&local, sizeof(local)) <0)
        perror("bind error"), exit(1);
    }

  if(ready(sock))
    return recvfrom(sock, pktbuf, pktlen, 0, &local, &n);
  else
    return 0;
}


static int dmxoed_watch(U16 port_in, int dmx_fd)
{
  U8        netbuf[0x5000];
  U8        pktbuf[0x800];
  int       port_status;
  int       pktlen = -1;
  time_t    ticks;

  bzero (netbuf, sizeof(netbuf));
  for(;;)
    {
      size_t len = 0;

      port_status = recv_udp(port_in, pktbuf, sizeof(pktbuf));
      if(port_status < 0)
        {
          perror("error receiving");
          return -1;
        }
      if(port_status > 0)
        {
          ticks = time((time_t*)NULL);
          pktlen = port_status;
          len = shownet_decode(pktbuf, pktlen, netbuf, sizeof(netbuf));
        }
      if(len > 0)
        {
          printf ("len=%i\r", len);
          fflush(stdout);
          lseek (dmx_fd, 0, SEEK_SET);
          write (dmx_fd, netbuf, len);
        }
    }
  return 0;
}

static void usage (char *name)
{
  fprintf (stderr, "Usage: %s [-p <port>] [-d <dmx-device>]\n", name);
  fprintf (stderr, "       %s [-h | -?]\n", name);
  fprintf (stderr, "Parameters:\n");
  fprintf (stderr, "  -p <port>       UDP Port to listen on. (2501)\n");
  fprintf (stderr, "  -d <dmx-device> DMX-Devide to write dmx-values to. (/dev/dmx)\n");
  fprintf (stderr, "  -h | -?         Print this help.\n");
}

static int dmx_fd = -1;
static void close_devices()
{
  if (dmx_fd!=-1)
    close(dmx_fd);
}

int main (int argc, char* argv[])
{
  int   port_in    = 2501;
  char *dmx_device = "/dev/dmx";
  int   i;

  for (i=1; i<argc; i++)
    {
      if (argv[i][0]=='-')
        {
          switch (argv[i][1])
            {
              case 'p': if ((++i) < argc)
                          {
                            port_in = atoi(argv[i]);
                          }
                        else
                          {
                            usage(argv[0]);
                            return 0;
                          }
                        break;
              case 'd': if ((++i) < argc)
                          {
                            dmx_device = argv[i];
                          }
                        else
                          {
                            usage(argv[0]);
                            return 0;
                          }
                        break;
              case '?': usage(argv[0]); return 0;
              case 'h': usage(argv[0]); return 0;
            }
        }
      else
        {
          usage(argv[0]);
          return 0;
        }
    }

  dmx_fd = open(dmx_device, O_WRONLY);
  if (dmx_fd==-1)
    {
      printf ("unable to open dmx device %s\n", dmx_device);
      exit(0);
    }
  atexit(close_devices);

  fprintf (stderr, "using dmx-device: %s\n", dmx_device);
  fprintf (stderr, "using udp-port  : %i\n", port_in);

  if (dmxoed_watch(port_in, dmx_fd) < 0)
    fprintf (stderr, "error while listening on udp-port %i\n", dmx_fd);

  return 0;
}

