
#ifndef __dmxoe_codec_h__
#define __dmxoe_codec_h__


#include <curses.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <sys/select.h>
#include <sys/time.h>

#ifdef __cplusplus
extern "C" {
#endif


#define min(a,b) ((a) < (b) ? (a) : (b))
#define max(a,b) ((a) > (b) ? (a) : (b))

typedef unsigned char  U8;
typedef unsigned short U16;
typedef unsigned int   U32;
typedef unsigned long  U64;

#define HEADER_SIZE 47
typedef struct {
  U16 version;
  U32 srcip;
  U16 slot[4];
  U16 length[4];
  U16 rle_offset[5];
  U16 sequence;

  U8  long_sig0[4];  /* first part of the unknown signature: $00,$00,$58,$02 */
  U8  devicename[8]; /* console1 */
  U8  long_sig1[1];  /* second part of the unknown signature: $0 */

  //  rle encoded dimmer data
  U8  data[0];
} __attribute__ ((packed)) header;

#define LONG_SIG "00 00 58 02  63 6F 6E 73  6F 6C 65 31  00"
#define LONG_SIG_ARRAY {0x00,0x00,0x58,0x02,0x63,0x6F,0x6E,0x73,0x6F,0x6C,0x65,0x31,0x00}
#define RLE_OFFSET_BIAS 11




extern size_t rle_encode(U8 *srcbuf, size_t srclen, U8 *dstbuf, size_t dstlen);
extern size_t shownet_encode (U8 pktbuf[], unsigned pktlen, U8 netbuf[], unsigned netlen);

extern size_t rle_decode(U8 srcbuf[], unsigned srclen, U8 dstbuf[], unsigned dstlen);
extern size_t shownet_decode(U8 pktbuf[], unsigned pktlen, U8 netbuf[], unsigned netlen);

extern void hex_dump(U8 buf[], unsigned len);

void display_header(header* phead);

bool ready(int sock);

int recv_udp(U16 port_in, U8 buf[], unsigned len);

void watch(U16 port_in);




#ifdef __cplusplus
};
#endif



class ShownetCodec
{
private:
private:
  U16  ThisVersion    = 1;
  U16  packetSequence = 1;
  U32  my_ip = (192<<24) | (168<<16) | (0<<8) | 49;


public:
  ShownetCodec  ();
  ~ShownetCodec ();

  U16    getPacketSequence ();
  long   getSlotLayout (int count, U16 *lengths);

  size_t rle_encode     (U8 *dstbuf, size_t dstlen, U8 *srcbuf, size_t srclen);
  size_t rle_decode     (U8 srcbuf[], size_t srclen, U8 dstbuf[], size_t dstlen);

  size_t shownet_encode (U8 dmxbuf[], size_t dmxlen, U8 netbuf[], size_t netlen);
  size_t shownet_decode (U8 pktbuf[], size_t pktlen, U8 netbuf[], size_t netlen);

};


#endif
